/****
 *
 * Ttulo: Traduzindo Nmeros de Telefone
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/03/2013
 * ltima modificao: 16/03/2013
 *
 * Descrio: Escreve na tela nmeros de telefones correspondentes
 *            a expresses contendo letras, '0', '1' ou '-'
 *
 * Entrada: Strings contendo letras, '0', '1' ou '-'
 *
 * Sada: Nmeros de telefone correspondentes aos strings
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* putchar() e printf()  */
#include <string.h>  /* strchr()              */
#include <ctype.h>   /* isalpha() e toupper() */
#include <stdlib.h>  /* exit()                */
#include "leitura.h" /* LeString()            */

/***************** Constantes Simblicas ******************/

   /* Tamanho mximo de um string que     */
   /* representa uma expresso telefnica */
#define MAX_EXPRESSAO 30

   /* Nmero mximo de caracteres associados a um dgito */
#define MAX_CAR_POR_DIGITO 4

/****************** Definies de Tipos *******************/

   /* Tipo de estrutura que representa uma correspondncia  */
   /* entre as letras de  um string e um dgito de telefone */
typedef struct {
           char letras[MAX_CAR_POR_DIGITO + 1];
           int  digito;    /* Dgito correspondente a  */
                           /* qualquer letra do string */
        } tCorrespondencia;

/************************* Aluses ************************/

extern void ApresentaEspacos(int nEspacos);
extern void ApresentaErro(const char *str,const char *erro);
extern int ChecaExpressao(const char *str);
extern void EscreveTelefone(const char *s);

/***************** Definies de Funes ******************/

/****
 *
 * ApresentaEspacos(): Escreve o nmero especificado
 *                     de espaos em branco na tela
 *
 * Parmetros:
 *      nEspacos (entrada) - nmero de espaos em branco
 *                           que sero escritos
 *
 * Retorno: Nenhum
 *
 ****/
void ApresentaEspacos(int nEspacos)
{
   int i;

      /* Escreve o nmero especificado de espaos na tela */
   for (i = 1; i <= nEspacos; ++i) {
      putchar(' '); /* Escreve um espao na tela */
   }
}

/****
 *
 * ApresentaErro(): Apresenta uma mensagem de erro
 *                  apontando para o caractere num
 *                  string que causou o erro
 *
 * Parmetros:
 *      str (entrada) - o string contendo o erro
 *      erro (entrada) - ponteiro para o caractere
 *                       causador do erro
 *
 * Retorno: Nenhum
 *
 ****/
void ApresentaErro(const char *str, const char *erro)
{
   int posicao;

      /* Determina o ndice do caractere que causou erro */
   posicao = erro - str;

      /* O ndice do caractere no pode ser negativo */
   if (posicao < 0) {
         /* Apresenta uma mensagem de erro padro */
      printf("\a\n\t>>> Erro nao indentificado!\n");

      return; /* Nada mais pode ser feito */
   }

      /* Apresenta o string com embelezamento */
      /* grfico e sonoro                     */
   printf("\a\n>>> Erro <<<\n\n%s\n", str);

      /**************************/
      /*                        */
      /* Indica o local do erro */
      /*                        */
      /**************************/

      /* Escreve a ponta da seta para cima */
   ApresentaEspacos(posicao);
   printf("^\n");

      /* Escreve a haste da seta */
   ApresentaEspacos(posicao);
   printf("|\n");

      /* Escreve a mensagem de erro */
   ApresentaEspacos(posicao);
   printf("-- Caractere espurio\n");
}

/****
 *
 * ChecaExpressao(): Verifica se uma expresso (string) que
 *                   representa um nmero telefnico  vlida
 *                   e, quando for o caso, indica na tela onde
 *                   se encontra o primeiro erro detectado
 *
 * Parmetros:
 *      str (entrada) - string que representa a expresso que
 *                      ser verificada
 *
 * Retorno: 1, se a expresso estiver OK; 0, em caso contrrio
 *
 * Observao: Uma expresso vlida contm apenas letras,
 *             '0', '1' e '-'
 *
 ****/
int ChecaExpressao(const char *str)
{
   int i;

      /* Examina cada caractere do string */
      /* e verifica se ele  vlido       */
   for (i = 0; str[i]; ++i){
      if (!isalpha(str[i])) {
         if ( str[i] != '0' && str[i] != '1'&&
              str[i] != '-') {
               /* Foi encontrado um caractere invlido. */
               /* Apresenta mensagem de erro informando */
               /* onde ele se encontra.                 */
            ApresentaErro(str, &str[i]);

            return 0; /* Caractere invlido encontrado */
         }
      }
   }

   return 1; /* No foi encontrado nenhum erro */
}

/****
 *
 * EscreveTelefone(): Escreve na tela o nmero de telefone
 *                    correspondente a um string vlido
 *
 * Parmetros:
 *      s (entrada) - string que representa a expresso cujo
 *                    telefone correspondente ser escrito
 *
 * Retorno: Nenhum
 *
 * Observaes:
 *     1.  Esta funo assume que o string  vlido
 *     2.  Se o correspondente a algum caractere no for
 *         escrito, o programa ser abortado
 *
 ****/
void EscreveTelefone(const char *s)
{
      /* O array a seguir contm as correspondncias    */
      /* entre letras e dgitos. O uso de static        */
      /* garante uma nica iniciao da varivel e o    */
      /* uso de const garante que ela no ser alterada */
   static const tCorrespondencia tabela[] = { {"ABC",  '2'},
                                              {"DEF",  '3'},
                                              {"GHI",  '4'},
                                              {"JKL",  '5'},
                                              {"MNO",  '6'},
                                              {"PQRS", '7'},
                                              {"TUV",  '8'},
                                              {"WXYZ", '9'}
                                            };
   int i,
       tamTabela, /* Tamanho da tabela de correspondncia */
       carEscrito; /* Indicar se o correspondente a um */
                   /* caractere do string foi escrito   */

      /* Calcula o tamanho da tabela de correspondncia */
   tamTabela = sizeof(tabela)/sizeof(tabela[0]);

   printf("\n\t>>> Telefone: "); /* Embelezamento */

   for (; *s; ++s) {
         /* O caractere corrente ainda no foi escrito */
      carEscrito = 0;

      if (isalpha(*s)) {
            /* O caractere corrente  uma letra cujo   */
            /* valor numrico ser procurado na tabela */
         for (i = 0; i < tamTabela; ++i) {
               /* Procura na tabela o caractere corrente */
               /* convertido em letra maiscula          */
            if ( strchr(tabela[i].letras, toupper(*s)) ) {
                  /* A letra foi encontrada e o nmero */
                  /* correspondente ser escrito       */
               putchar(tabela[i].digito);

               carEscrito = 1; /* O caractere foi escrito */
            }
         }
      } else if (*s == '0' || *s == '1' || *s == '-'){
            /* Se o caractere for '0', '1' ou */
            /* '-', ele  escrito como ele   */
         putchar(*s);

         carEscrito = 1; /* O caractere foi escrito */
      }

         /* Se no foi escrita a correspondncia de   */
         /* algum caractere, o programa ser abortado */
      if (!carEscrito) {
         printf( "\nERRO: O caractere %c nao foi escrito\n",
                 *s );
         exit(1);
      }
   }
}

/****
 *
 * main(): Escreve na tela nmeros de telefones correspondentes a
 *         expresses contendo letras, '0', '1' ou '-'
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char str[MAX_EXPRESSAO + 1];
   int  excesso;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa apresenta numeros de telefone "
           "que\n\t>>> correspondem a expressoes com letras, "
           "'0', '1' ou '-'" );

      /* O lao termina quando o usurio */
      /* digitar apenas [ENTER]          */
   while (1) {
         /* L a expresso */
      printf( "\n\nIntroduza uma expressao ou "
              "[ENTER] para encerrar:\n\t> " );
      excesso = LeString(str, MAX_EXPRESSAO + 1);

      if (!*str) { /* Checa se o string  vazio */
         break; /* Usurio digitou apenas [ENTER] */
      }

         /* Verifica se foram digitados caracteres demais */
      if (excesso) {
         printf( "\n\t>>> Voce digitou caracteres demais."
                 "\nO maximo permitido sao %d caracteres\n",
                 MAX_EXPRESSAO );

         continue; /* Salta o restante do corpo do lao */
      }

         /* Verifica se a expresso  vlida */
      if (!ChecaExpressao(str)) {
            /* A expresso no  vlida */
         continue; /* Salta o resto do lao */
      }

         /* Apresenta na tela o nmero de telefone */
         /* correspondente  expresso introduzida */
      EscreveTelefone(str);
   } /* while */

   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa apresenta numeros de telefone que
        >>> correspondem a expressoes com letras, '0', '1' ou '-'

Introduza uma expressao ou [ENTER] para encerrar:
        > 1-ab2c-def

>>> Erro <<<

1-ab2c-def
    ^
    |
    -- Caractere espurio


Introduza uma expressao ou [ENTER] para encerrar:
        > 1-abc-defg-uvzw

        >>> Telefone: 1-222-3334-8899

Introduza uma expressao ou [ENTER] para encerrar:
        > farmacia

        >>> Telefone: 32762242

Introduza uma expressao ou [ENTER] para encerrar:
        > [ENTER]

        >>> Obrigado por usar este programa.

**********************************************************/
