/****
 *
 * Ttulo: Comparando Dois Arquivos
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 23/06/2012
 * ltima modificao: 23/06/2012
 *
 * Descrio: Compara dois arquivos byte a byte
 *
 * Entrada: Nomes dos dois arquivos
 *
 * Sada: Uma mensagem informando se os arquivos so iguais ou
 *        diferentes ou se ocorreu erro de leitura em algum arquivo
 *
 ****/

#include <stdio.h>   /* Entrada e sada */
#include <stdlib.h>  /* Funo exit()   */

/****
 *
 * FechaArquivo(): Tenta fechar um arquivo e, quando isso
 *                 no  possvel, aborta o programa
 *
 * Parmetros:
 *      stream (entrada) - stream associado ao arquivo
 *      nomeArq (entrada) - nome do arquivo ou NULL
 *
 * Retorno: Nada
 *
 * Observao: Se o segundo parmetro no for NULL, o nome
 *             do programa aparece na mensagem de erro
 *
 ****/
void FechaArquivo(FILE *stream, const char *nomeArq)
{
   /***************************************************/
   /*                                                 */
   /* Se fclose() retornar um valor diferente de zero */
   /* ocorreu algum erro na tentativa de fechamento   */
   /* do arquivo. Neste caso, apresenta-se mensagem   */
   /* de erro e aborta-se o programa.                 */
   /*                                                 */
   /***************************************************/

   if (fclose(stream)) { /* Erro de fechamento */
      fprintf( stderr, "\a\n>>> Ocorreu erro no fechamento"
              " do arquivo %s.\n>>> O programa sera' "
              "encerrado.\n", nomeArq ? nomeArq : "");
      exit(1); /* Aborta o programa */
   }
}

/****
 *
 * main(): Compara dois arquivos byte a byte. Os nomes dos
 *         arquivos so argumentos de linha de comando.
 *
 * Parmetros:
 *     argc (entrada) - Nmero de argumentos de linha de comando
 *     argv (entrada) - Array de strings presentes na linha de
 *                      comando quando o programa  executado
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(int argc, char *argv[])
{
   int  contaBytes = 0, /* Conta os byte comparados */
                        /* em cada arquivo          */
        diferentes = 0, /* Assumir 1, se forem encontrados */
                        /* dois bytes  diferentes           */
        erro1 = 0, /* Indicador de erro no arquivo 1 */
        erro2 = 0, /* Indicador de erro no arquivo 2 */
        byte1, /* Byte lido no arquivo 1 */
        byte2; /* Byte lido no arquivo 2 */
   FILE *stream1, /* Stream associado ao arquivo 1 */
        *stream2; /* Stream associado ao arquivo 2 */

      /* O usurio deve informar os nomes dos arquivos */
      /* que sero comparados na linha de comando      */
   if (argc != 3) {
      printf( "\n\t>>> Este programa deve ser usado assim:"
              "\n\t\t> %s nome-de-arquivo1 nome-de-arquivo2\n",
              argv[0] );
      return 1; /* Usurio no sabe usar o programa */
   }

      /* Tenta abrir o primeiro arquivo */
      /* em modo binrio para leitura   */
   stream1 = fopen(argv[1], "rb");

      /* Se o arquivo no pode ser aberto */
      /* nada mais pode ser feito         */
   if (!stream1) {
      printf( "\n\t>>> Nao foi possivel abrir o arquivo "
              "\"%s\"\n", argv[1] );
      return 1; /* Primeiro arquivo no foi aberto */
   }

      /* Tenta abrir o segundo arquivo */
      /* em modo binrio para leitura  */
   stream2 = fopen(argv[2], "rb");

      /* Se o arquivo no pode ser aberto */
      /* nada mais pode ser feito         */
   if (!stream2) {
      printf( "\n\t>>> Nao foi possivel abrir o arquivo \"%s\"\n",
              argv[2] );

      fclose(stream1); /* Fecha primeiro arquivo */
                       /* antes de retornar      */
      return 1; /* Segundo arquivo no foi aberto */
   }

      /*                                                     */
      /* Condies de parada do lao while a seguir:         */
      /*  - A leitura atinge o final de um dos arquivos      */
      /*  - Ocorre erro de leitura em um dos arquivos        */
      /*  - So lidos dois bytes correspondentes que diferem */
      /*                                                     */

   while ( !feof(stream1) && !feof(stream2) ) {
      byte1 = fgetc(stream1); /* L um byte no arquivo 1 */
      byte2 = fgetc(stream2); /* L um byte no arquivo 2 */

      contaBytes++; /* Mais um byte lido em cada arquivo */

         /* Se ocorreu erro de leitura no */
         /* arquivo 1, encerra o lao     */
      if (ferror(stream1)) {
         erro1 = 1; /* Se no desejar usar goto nem   */
                    /* duplicar cdigo, essa varivel */
                    /*  necessria (v. abaixo)       */
         break;
      }

         /* Se ocorreu erro de leitura no */
         /* arquivo 2, encerra o lao     */
      if (ferror(stream2)) {
         erro2 = 1; /* Se no desejar usar goto nem duplicar */
                    /* cdigo, essa varivel  necessria    */
         break;
      }

         /* Se a leitura chega ao final num arquivo */
         /* antes de chegar ao final no outro, os   */
         /* arquivo so obviamente diferentes       */
      diferentes = ( feof(stream1) && !feof(stream2) ) ||
                   ( !feof(stream1) && feof(stream2) );

         /* Se j foi constatado que os arquivos so */
         /* diferentes, encerra-se a comparao      */
      if (diferentes) {
         break;
      }

         /* Se os ltimos bytes lidos forem diferentes, */
         /* os arquivos so diferentes                  */
      if (byte1 != byte2) {
         diferentes = 1;
         break;
      }
   }

      /* Fecha os arquivos */
   FechaArquivo(stream1, argv[1]);
   FechaArquivo(stream2, argv[2]);

      /* Se ocorreu erro no arquivo 1, informa o    */
      /* usurio. No  mais possvel usar ferror() */
      /* pois os arquivos j foram fechados. Isso   */
      /* justifica o uso da varivel erro1.         */
   if (erro1) {
      printf( "\n\t>>> Erro de leitura no arquivo \"%s\"\n",
              argv[1] );
      return 1;
   }

      /* Se ocorreu erro no arquivo 2, informa o    */
      /* usurio. No  mais possvel usar ferror() */
      /* pois os arquivos j foram fechados. Isso   */
      /* justifica o uso da varivel erro2.         */
   if (erro2) {
      printf( "\n\t>>> Erro de leitura no arquivo \"%s\"\n",
              argv[2] );
      return 1;
   }

      /* Se os arquivos forem diferentes, informa  */
      /* o ndice dos respectivos bytes em que foi */
      /* detectada a primeira diferena. Aqui, os  */
      /* bytes so indexados a partir de 1.        */
   if (diferentes) {
      printf( "\n\t>>> Os arquivos diferem no byte numero %d\n",
              contaBytes );
   } else { /* No foi detectada diferena */
      printf("\n\t>>> Os arquivos sao identicos\n");
   }

   return 0;
}

/******************* Exemplos de Execuo ******************

[O nome do arquivo executvel  compara]

C:\Programas>Compara

        >>> Este programa deve ser usado assim:
                > Compara nome-de-arquivo1 nome-de-arquivo2

C:\Programas>Compara Tudor.txt TudorBK.txt

        >>> Os arquivos diferem no byte numero 38

C:\Programas>Compara Tudor.txt Tudor.txt

        >>> Os arquivos sao identicos

***********************************************************/
