/****
 *
 * Ttulo: Nmero de Registros de um Arquivo Binrio
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 19/05/2012
 * ltima modificao: 19/05/2012
 *
 * Descrio: Calcula nmero de registros de um determinado tipo
 *            de um arquivo binrio
 *
 * Entrada: No h
 *
 * Sada: v. Exemplo de Execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h> /* Entrada e sada */

/***************** Constantes Simblicas ******************/

#define MAX_NOME 20 /* Nmero mximo de caracteres num nome */
#define TAM_MATR  4 /* Nmero de dgitos numa matrcula */
#define NOME_ARQUIVO_BIN "Tudor.bin" /* Arquivo binrio */

/****************** Definies de Tipos *******************/

typedef  struct {
            char    nome[MAX_NOME + 1];
            char    matr[TAM_MATR + 1];
            double  n1, n2;
         } tAluno;

/***************** Definies de Funes ******************/

/****
 *
 * TamanhoDeArquivo(): Calcula o tamanho de um arquivo lendo cada
 *                     byte que o constitui e contando quantos
 *                     bytes so lidos
 *
 * Parmetros:
 *    stream (entrada) - stream binrio associado ao arquivo cujo
 *                       tamanho ser calculado
 *
 * Retorno: -1, se ocorrer algum erro.
 *          O tamanho do arquivo, em caso contrrio.
 *
 * Observaes:
 *     1. O arquivo deve estar aberto em modo binrio que permite
 *        leitura
 *     2. A posio atual do indicador de posio do arquivo 
 *        preservada
 *
 ****/
int TamanhoDeArquivo(FILE *stream)
{
   int posicaoAtual, tamanho = 0;

      /* Guarda a posio atual do indicador de posio */
      /* do arquivo para que ele possa ser restaurado   */
   posicaoAtual = ftell(stream);

      /* Verifica se ftell() foi bem sucedida */
   if (posicaoAtual < 0) {
      return -1; /* A chamada de ftell() falhou */
   }

      /* Assegura leitura a partir do incio do arquivo */
   if ( fseek(stream, 0, SEEK_SET) ) {
         /* O movimento do indicador de posio no foi   */
         /* possvel. Neste caso, no precisa restaur-lo */
      return -1;
   }

      /* O lao encerra quando o final do arquivo for */
      /* atingido ou se ocorrer erro de leitura       */
   while (1){
         /* L e descarta o prximo byte */
      fgetc(stream);

         /* Checa se o arquivo j foi inteiramente lido */
      if (feof(stream)) {
         break; /* Encerra lao */
      }

         /* Verifica se ocorreu erro de leitura */
      if (ferror(stream)) {
            /* Restaura a posio original do indicador */
            /* de posio do arquivo  antes de retornar */
         fseek(stream, posicaoAtual, SEEK_SET);
         return -1; /* Ocorreu erro de leitura */
      }

      ++tamanho; /* Mais um byte foi lido */
   }

      /* Restaura a posio original do indicador de posio */
   if ( fseek(stream, posicaoAtual, SEEK_SET) ) {
      return -1; /* O movimento no foi possvel */
   }

   return tamanho;
}

/****
 *
 * NumeroDeRegistros(): Calcula o nmero de registros (i.e.,
 *                      parties de tamanho especificado) de um
 *                      arquivo binrio
 *
 * Parmetros:
 *    stream (entrada) - stream binrio associado ao arquivo cujo
 *                       tamanho ser calculado
 *    tamRegistro (entrada) - tamanho de cada partio do arquivo
 *
 * Retorno: O nmero de parties do arquivo, se no ocorrer
 *          nenhum erro, ou -1, em caso contrrio.
 *
 * Observao: O arquivo deve estar aberto em modo binrio que
 *             permite leitura
 *
 ****/
int NumeroDeRegistros( FILE *stream, int tamRegistro )
{
   int    nBytes = 0; /* Nmero de bytes do arquivo */

      /* Calcula o nmero de bytes do arquivo */
   nBytes = TamanhoDeArquivo(stream);

      /* Verifica se ocorreu erro na contagem de bytes */
   if (nBytes < 0) {
      return -1; /* Erro na contagem de bytes */
   }

      /* O nmero de registros  o nmero de bytes do arquivo */
      /* dividido pelo nmero de bytes de cada registro       */
   return nBytes/tamRegistro;
}

/****
 *
 * main(): Assume que um arquivo binrio  constitudo de registros
 *         de um mesmo tipo e calcula o nmero de registros que
 *         compem o arquivo
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   FILE  *stream; /* Stream associado ao arquivo */
   int    nRegistros; /* Nmero de registros do arquivo */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa apresenta o numero de registros"
           "\n\t>>> do arquivo \"%s\"\n", NOME_ARQUIVO_BIN );

      /* Tenta abrir arquivo binrio */
   stream = fopen(NOME_ARQUIVO_BIN, "rb");

      /* Se o arquivo no pode ser aberto, */
      /* nada mais pode ser feito          */
   if (!stream) {
      printf( "\n\t>>> Arquivo \"%s\" nao pode ser aberto",
              NOME_ARQUIVO_BIN );
      return 1; /* Operao mal sucedida */
   }

      /* Calcula o nmero de registros do arquivo */
   nRegistros = NumeroDeRegistros( stream, sizeof(tAluno) );

      /* Verifica se ocorreu erro na operao */
   if (nRegistros < 0) {
      printf( "\n\t>>> Ocorreu um erro no processamento do "
              "\n\t>>> arquivo \"%s\"", NOME_ARQUIVO_BIN );
      return 1;
   }

      /* Apresenta o resultado */
   printf( "\n\t>>> Numero de registros do arquivo \"%s\":"
           " %d\n", NOME_ARQUIVO_BIN, nRegistros );

   fclose(stream); /* Fecha o arquivo */

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa apresenta o numero de registros
        >>> do arquivo "Tudor.bin"

        >>> Numero de registros do arquivo "Tudor.bin": 7

***********************************************************/
